﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading.Tasks;

namespace EchoTcpServer
{
    public class EchoServer
    {
        public int Port { get; }
        public IPAddress LocalAddress { get; }

        private TcpListener _server;
        private volatile bool _listening = true;
        public EchoServer(int port)
        {
            Port = port;
            LocalAddress = IPAddress.Parse("127.0.0.1");
            //LocalAddress = IPAddress.Loopback;
            //IPAddress localAddress = IPAddress.Any; // 0.0.0.0
            //string hostName = Dns.GetHostName();
            //IPHostEntry s = Dns.GetHostEntry(hostName);
            //LocalAddress = s.AddressList[0];
            //IPAddress localAddress = Dns.GetHostEntry("127.0.0.1");
        }

        public void Start()
        {
            // https://msdn.microsoft.com/en-us/library/system.net.sockets.tcplistener.aspx
            _server = new TcpListener(LocalAddress, Port);
            _server.Start();
            Console.WriteLine("Echo TCP server started on " + LocalAddress + " port " + Port);
            while (true)
            {
                // busy waiting, ugly!
                //if (!_server.Pending()) //checking for pending connections 
                //{
                //    Thread.Sleep(50);
                //    continue; 
                //}
                Console.WriteLine("Waiting for connection ...");
                TcpClient client = _server.AcceptTcpClient();
                Console.WriteLine("Connected!");
                Task.Run(() => DoIt(client));
            }
        }

        public void Stop()
        {
            _listening = false;
            _server.Stop();
        }

        private static void DoIt(TcpClient client)
        {
            try
            {
                NetworkStream stream = client.GetStream();
                StreamReader reader = new StreamReader(stream);
                string request = reader.ReadLine(); // echo is a line based protocol
                string response = request; // this is echo!
                StreamWriter writer = new StreamWriter(stream);
                writer.WriteLine(response);
                writer.Flush();
            }
            catch (IOException ex)
            {
                Console.Error.WriteLine(ex.Message);
            }
            finally
            {
                client.Close();
            }
        }
    }
}
